      subroutine read_control
C
C Read molecular data
C
      implicit none
      include 'DIMENSIONS'
      include 'sizesclu.dat'
      include 'COMMON.IOUNITS'
      include 'COMMON.TIME1'
      include 'COMMON.SBRIDGE'
      include 'COMMON.CONTROL'
      include 'COMMON.CLUSTER'
      include 'COMMON.CHAIN'
      include 'COMMON.HEADER'
      include 'COMMON.FFIELD'
      include 'COMMON.FREE'
      character*320 controlcard,ucase
#ifdef MPL
      include 'COMMON.INFO'
#endif
      integer i

      read (INP,'(a80)') titel
      call card_concat(controlcard)
      call readi(controlcard,'NRES',nres,0)
      call readi(controlcard,'RESCALE',rescale_mode,2)
      call readi(controlcard,'PDBOUT',outpdb,0)
      call readi(controlcard,'MOL2OUT',outmol2,0)
      refstr=(index(controlcard,'REFSTR').gt.0)
      unres_pdb=(index(controlcard,'UNRES_PDB').gt.0)
      energy_dec=(index(controlcard,'ENERGY_DEC').gt.0)
      pdbref=(index(controlcard,'PDBREF').gt.0)
      iscode=index(controlcard,'ONE_LETTER')
      tree=(index(controlcard,'MAKE_TREE').gt.0)
      min_var=(index(controlcard,'MINVAR').gt.0)
      plot_tree=(index(controlcard,'PLOT_TREE').gt.0)
      punch_dist=(index(controlcard,'PUNCH_DIST').gt.0)
      call readi(controlcard,'NCUT',ncut,1)
      call readi(controlcard,'NSTART',nstart,0)
      call readi(controlcard,'NEND',nend,0)
      call reada(controlcard,'ECUT',ecut,10.0d0)
      call reada(controlcard,'PROB',prob_limit,0.99d0)
      lgrp=(index(controlcard,'LGRP').gt.0)
      caonly=(index(controlcard,'CA_ONLY').gt.0)
      print_dist=(index(controlcard,'PRINT_DIST').gt.0)
      call multreada(controlcard,'CUTOFF',rcutoff,ncut,-1.0d0)
      call readi(controlcard,'IOPT',iopt,2) 
      lside = index(controlcard,"SIDE").gt.0
      efree = index(controlcard,"EFREE").gt.0
      call readi(controlcard,'NTEMP',nT,1)
      call multreada(controlcard,'TEMPER',beta_h,nT,300.0d0)
      write (iout,*) "nT",nT
      do i=1,nT
        beta_h(i)=1.0d0/(1.987D-3*beta_h(i))
      enddo
      write (iout,*) 'beta_h',(beta_h(i),i=1,nT)
      lprint_cart=index(controlcard,"PRINT_CART") .gt.0
      lprint_int=index(controlcard,"PRINT_INT") .gt.0
      if (min_var) iopt=1
      return
      end
c--------------------------------------------------------------------------
      subroutine molread
C
C Read molecular data.
C
      implicit none
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
      include 'COMMON.GEO'
      include 'COMMON.VAR'
      include 'COMMON.INTERACT'
      include 'COMMON.LOCAL'
      include 'COMMON.NAMES'
      include 'COMMON.CHAIN'
      include 'COMMON.FFIELD'
      include 'COMMON.SBRIDGE'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.CONTACTS'
      include 'COMMON.TIME1'
#ifdef MPL
      include 'COMMON.INFO'
#endif
      character*4 sequence(maxres)
      character*800 weightcard
      integer rescode,sugarcode
      double precision x(maxvar)
      integer itype_pdb(maxres)
      logical seq_comp
      integer i,j
C
C Body
C
C Read weights of the subsequent energy terms.
      call card_concat(weightcard)
      call reada(weightcard,'WVDWPP',wvdwpp,1.0D0)
      call reada(weightcard,'WELPP',welpp,1.0d0)
      call reada(weightcard,'WVDWPSB',wvdwpsb,1.0d0)
      call reada(weightcard,'WELPSB',welpsb,1.0D0)
      call reada(weightcard,'WVDWSB',wvdwsb,1.0d0)
      call reada(weightcard,'WELSB',welsb,1.0D0)
      call reada(weightcard,'WBOND',wbond,1.0D0)
      call reada(weightcard,'WANG',wang,1.0D0)
      call reada(weightcard,'WSBLOC',wsbloc,1.0D0)
      call reada(weightcard,'WTOR',wtor,1.0D0)
      call reada(weightcard,'WTORD',wtor_d,1.0D0)
      call reada(weightcard,'WCORR',wcorr,1.0D0)
      call reada(weightcard,'WCORR3',wcorr3,1.0D0)
      call reada(weightcard,'WSTRAIN',wstrain,1.0D0)
      call reada(weightcard,'SCAL14',scal14,0.4D0)
      call reada(weightcard,'SCALSCP',scalscp,1.0D0)
      call reada(weightcard,'CUTOFF',cutoff_corr,7.0d0)
      call reada(weightcard,'DELT_CORR',delt_corr,0.5d0)
      call reada(weightcard,'TEMP0',temp0,300.0d0)
      call reada(weightcard,'DISTCHAINMAX',distchainmax,30.0d0)
      write (iout,*) "distchainmax",distchainmax
      if (index(weightcard,'SOFT').gt.0) ipot=6
      weights(1)=wvdwpp
      weights(2)=welpp
      weights(3)=wvdwpsb
      weights(4)=welpsb
      weights(5)=wvdwsb
      weights(6)=welsb
      weights(7)=wbond
      weights(8)=wang
      weights(9)=wsbloc
      weights(10)=wtor
      weights(11)=wtor_d
      weights(12)=wcorr
      weights(13)=wcorr3
      weights(14)=wstrain
      weights(15)=scal14
      write (iout,10) wvdwpp,welpp,wvdwpsb,welpsb,wvdwsb,welsb,
     &  wbond,wang,wsbloc,wtor,wtor_d,wcorr,wcorr3,wstrain
   10 format (/'Energy-term weights (unscaled):'//
     & 'WVDWPP= ',f10.6,' (P-P VDW)'/
     & 'WELPP=  ',f10.6,' (P-P ELEC)'/
     & 'WVDWPSB=',f10.6,' (P-SB VDW)'/
     & 'WELPSB= ',f10.6,' (P-SB ELEC)'/
     & 'WVDWSB= ',f10.6,' (SB-SB VDW)'/
     & 'WELSB=  ',f10.6,' (SB-SB ELEC)'/
     & 'WBOND=  ',f10.6,' (stretching)'/
     & 'WANG=   ',f10.6,' (bending)'/
     & 'WSBLOC= ',f10.6,' (SB local)'/
     & 'WTOR=   ',f10.6,' (torsional)'/
     & 'WTORD=  ',f10.6,' (double torsional)'/
     & 'WSTRAIN=',f10.6,' (distance constraints)'/
     & 'WCORR=  ',f10.6,' (multi-body)'/
     & 'WCORR3= ',f10.6,' (multi-body 3rd order)')
       write (iout,'(a,f8.3)')
     &  'Scaling factor of 1,4 SC-p interactions:',scal14
       write (iout,'(a,f8.3)')
     &  'General scaling factor of SC-p interactions:',scalscp
      r0_corr=cutoff_corr-delt_corr
      do i=1,ntyp
        aad(i)=scalscp*aad(i)
        bad(i)=scalscp*bad(i)
      enddo
      call flush(iout)
      print *,'indpdb=',indpdb,' pdbref=',pdbref

C Read sequence of the nucleic acid
      read (inp,'(40a2)') (sequence(i),i=1,nres)
C Convert sequence to numeric code
      do i=1,nres
        istype(i)=sugarcode(sequence(i)(1:1),i)
        itype(i)=rescode(i,sequence(i)(2:2))
      enddo

      print *,nres
      print '(20i4)',(itype(i),i=1,nres)

      nnt=2
      nct=nres-1
      if (nstart.eq.0) nstart=nnt
      if (nend.eq.0) nend=nct
      nres0=nres
      call init_int_table
      call setup_var
      return
      end
c-----------------------------------------------------------------------------
      logical function seq_comp(itypea,itypeb,length)
      implicit none
      integer length,itypea(length),itypeb(length)
      integer i
      do i=1,length
        if (itypea(i).ne.itypeb(i)) then
          seq_comp=.false.
          return
        endif
      enddo
      seq_comp=.true.
      return
      end
c----------------------------------------------------------------------------
      subroutine read_angles(kanal,*)
      implicit none
      include 'DIMENSIONS'
      include 'COMMON.GEO'
      include 'COMMON.VAR'
      include 'COMMON.CHAIN'
      include 'COMMON.IOUNITS'
      integer i,kanal
      read (kanal,*,err=10,end=10) (theta(i),i=3,nres)
      read (kanal,*,err=10,end=10) (phi(i),i=4,nres)
      read (kanal,*,err=10,end=10) (alph(i),i=2,nres-1)
      read (kanal,*,err=10,end=10) (omeg(i),i=2,nres-1)
      do i=1,nres
        theta(i)=deg2rad*theta(i)
        phi(i)=deg2rad*phi(i)
        alph(i)=deg2rad*alph(i)
        omeg(i)=deg2rad*omeg(i)
      enddo
      return
   10 return1
      end
c----------------------------------------------------------------------------
      subroutine reada(rekord,lancuch,wartosc,default)
      implicit none
      character*(*) rekord,lancuch
      double precision wartosc,default
      integer ilen,iread
      external ilen
      iread=index(rekord,lancuch)
      if (iread.eq.0) then
        wartosc=default 
        return
      endif   
      iread=iread+ilen(lancuch)+1
      read (rekord(iread:),*) wartosc
      return
      end
c----------------------------------------------------------------------------
      subroutine multreada(rekord,lancuch,tablica,dim,default)
      implicit none
      integer dim,i
      double precision tablica(dim),default
      character*(*) rekord,lancuch
      integer ilen,iread
      external ilen
      do i=1,dim
        tablica(i)=default 
      enddo
      iread=index(rekord,lancuch)
      if (iread.eq.0) return
      iread=iread+ilen(lancuch)+1
      read (rekord(iread:),*,end=10,err=10) (tablica(i),i=1,dim)
   10 return
      end
c----------------------------------------------------------------------------
      subroutine readi(rekord,lancuch,wartosc,default)
      implicit none
      character*(*) rekord,lancuch
      integer wartosc,default
      integer ilen,iread
      external ilen
      iread=index(rekord,lancuch)
      if (iread.eq.0) then
        wartosc=default 
        return
      endif   
      iread=iread+ilen(lancuch)+1
      read (rekord(iread:),*) wartosc
      return
      end
c----------------------------------------------------------------------------
      subroutine card_concat(card)
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
      character*(*) card
      character*80 karta,ucase
      external ilen
      read (inp,'(a)') karta
      karta=ucase(karta)
      card=' '
      do while (karta(80:80).eq.'&')
        card=card(:ilen(card)+1)//karta(:79)
        read (inp,'(a)') karta
        karta=ucase(karta)
      enddo
      card=card(:ilen(card)+1)//karta
      return
      end
c----------------------------------------------------------------------------
      subroutine openunits
      implicit none
      include 'DIMENSIONS'    
#ifdef MPI
      include "mpif.h"
      character*3 liczba
      include "COMMON.MPI"
#endif
      include 'COMMON.IOUNITS'
      include 'COMMON.CONTROL'
      integer lenpre,lenpot,ilen
      external ilen
      character*16 cformat,cprint
      character*16 ucase
      integer lenint,lenout
      call getenv('INPUT',prefix)
      call getenv('OUTPUT',prefout)
      call getenv('INTIN',prefintin)
      call getenv('COORD',cformat)
      call getenv('PRINTCOOR',cprint)
      call getenv('SCRATCHDIR',scratchdir)
      from_bx=.true.
      from_cx=.false.
      if (index(ucase(cformat),'CX').gt.0) then
        from_cx=.true.
        from_bx=.false.
      endif
      from_cart=.true.
      lenpre=ilen(prefix)
      lenout=ilen(prefout)
      lenint=ilen(prefintin)
C Get the names and open the input files
      open (inp,file=prefix(:ilen(prefix))//'.inp',status='old')
#ifdef MPI
      write (liczba,'(bz,i3.3)') me
      outname=prefout(:lenout)//'_clust.out_'//liczba
#else
      outname=prefout(:lenout)//'_clust.out'
#endif
      if (from_bx) then
        intinname=prefintin(:lenint)//'.bx'
      else if (from_cx) then
        intinname=prefintin(:lenint)//'.cx'
      else
        intinname=prefintin(:lenint)//'.int'
      endif
      rmsname=prefintin(:lenint)//'.rms'
      open (jplot,file=prefout(:ilen(prefout))//'.tex',
     &       status='unknown')
      open (jrms,file=rmsname,status='unknown')
      open(iout,file=outname,status='unknown')
C Get parameter filenames and open the parameter files.
      call getenv('BONDPAR',bondname)
      open (ibond,file=bondname,status='old')
      call getenv('THETPAR',thetname)
      open (ithep,file=thetname,status='old')
      call getenv('ROTPAR',rotname)
      open (irotam,file=rotname,status='old')
      call getenv('TORPAR',torname)
      open (itorp,file=torname,status='old')
      call getenv('TORDPAR',tordname)
      open (itordp,file=tordname,status='old')
      call getenv('SIDEPAR',sidename)
      open (isidep,file=sidename,status='old')
      call getenv('SIDEP',sidepname)
      open (isidep1,file=sidepname,status="old")
#ifndef OLDSCP
C
C 8/9/01 In the newest version SCp interaction constants are read from a file
C Use -DOLDSCP to use hard-coded constants instead.
C
      call getenv('SCPPAR',scpname)
      open (iscpp,file=scpname,status='old')
#endif
      return
      end
