#ifdef MPI
      subroutine init_task
      implicit real*8 (a-h,o-z)
      include 'DIMENSIONS'
      include 'mpif.h'
      include 'COMMON.SETUP'
      include 'COMMON.CONTROL'
      include 'COMMON.IOUNITS'
c      real*8 text1 /'group_i '/,text2/'group_f '/,
c     & text3/'initialb'/,text4/'initiale'/,
c     & text5/'openb'/,text6/'opene'/
      integer cgtasks(0:max_cg_procs)
      character*3 cfgprocs 
      integer cg_size,fg_size
c  start parallel processing
      print *,'Initializing MPI'
      call mpi_init(ierr)
      if (ierr.ne.0) then
        print *, ' cannot initialize MPI'
        stop
      endif
c  determine # of nodes and current node
      call MPI_Comm_rank( MPI_COMM_WORLD, me, ierr )
      if (ierr.ne.0) then
        print *, ' cannot determine rank of all processes'
        call MPI_Finalize( MPI_COMM_WORLD, IERR )
        stop
      endif
      call MPI_Comm_size( MPI_Comm_world, nodes, ierr )
      if (ierr.ne.0) then
        print *, ' cannot determine number of processes'
        stop
      endif
      Nprocs=nodes
      MyRank=me
C Determine the number of "fine-grain" tasks
      call getenv_loc("FGPROCS",cfgprocs)
      read (cfgprocs,'(i3)') nfgtasks
      if (nfgtasks.eq.0) nfgtasks=1
      print *,"Processor",me," nfgtasks",nfgtasks
      if (nfgtasks.eq.1) then
        CG_COMM = MPI_COMM_WORLD
        fg_size=1
        fg_rank=0
      else
        nodes=nprocs/nfgtasks
        if (nfgtasks*nodes.ne.nprocs) then
          write (*,'(a)') 'ERROR: Number of processors assigned',
     &     ' to coarse-grained tasks must be divisor',
     &     ' of the total number of processors.'
          call MPI_Finalize( MPI_COMM_WORLD, IERR )
          stop
        endif
C Put the ranks of coarse-grain processes in one table and create
C the respective communicator. The processes with ranks "in between" 
C the ranks of CG processes will perform fine graining for the CG
C process with the next lower rank.
        do i=0,nprocs-1,nfgtasks
          cgtasks(i/nfgtasks)=i
        enddo
        print*,"Processor",me," cgtasks",(cgtasks(i),i=0,nodes-1)
        print "(a,i5,a)","Processor",myrank," Before MPI_Comm_group"
c        call memmon_print_usage()
        call MPI_Comm_group(MPI_COMM_WORLD,world_group,IERR)
        call MPI_Group_incl(world_group,nodes,cgtasks,cg_group,IERR)
        call MPI_Comm_create(MPI_COMM_WORLD,cg_group,CG_COMM,IERR)
        call MPI_Group_rank(cg_group,me,ierr)
        call MPI_Group_free(world_group,ierr)
        call MPI_Group_free(cg_group,ierr)
        print "(a,i5,a)","Processor",myrank," After MPI_Comm_group"
c        call memmon_print_usage()
        if (me.ne.MPI_UNDEFINED) call MPI_Comm_Rank(CG_COMM,me,ierr)
        print *," Processor",myrank," CG rank",me
C Create communicators containig processes doing "fine grain" tasks. 
C The processes within each FG_COMM should have fast communication.
        kolor=MyRank/nfgtasks
        key=mod(MyRank,nfgtasks)
        call MPI_Comm_split(MPI_COMM_WORLD,kolor,key,FG_COMM,ierr)
        call MPI_Comm_size(FG_COMM,fg_size,ierr)
        if (fg_size.ne.nfgtasks) then
          write (*,*) "OOOOps... the number of fg tasks is",fg_size,
     &      " but",nfgtasks," was requested. MyRank=",MyRank
        endif
        call MPI_Comm_rank(FG_COMM,fg_rank,ierr)
      endif
      if (fg_rank.eq.0) then
      write (*,*) "Processor",MyRank," out of",nprocs,
     & " rank in CG_COMM",me," size of CG_COMM",nodes,
     & " size of FG_COMM",fg_size
      else
      write (*,*) "Processor",MyRank," out of",nprocs,
     & " rank in FG_COMM",fg_rank," size of FG_COMM",fg_size
      endif
C Initialize other variables.
      print '(a)','Before initialize'
c      call memmon_print_usage()
      call initialize
      print '(a,i5,a)','Processor',myrank,' After initialize'
c      call memmon_print_usage()
C Open task-dependent files.
      print '(a,i5,a)','Processor',myrank,' Before openunits'
c      call memmon_print_usage()
      call openunits
      print '(a,i5,a)','Processor',myrank,' After openunits'
c      call memmon_print_usage()
      if (me.eq.king .or. fg_rank.eq.0 .and. .not. out1file) 
     &  write (iout,'(80(1h*)/a/80(1h*))') 
     & 'United-residue force field calculation - parallel job.'
      print *,"Processor",myrank," exited OPENUNITS"
      return
      end
C-----------------------------------------------------------------------------
      subroutine finish_task
      implicit real*8 (a-h,o-z)
      include 'DIMENSIONS'
      include 'mpif.h'
      include 'COMMON.SETUP'
      include 'COMMON.CONTROL'
      include 'COMMON.REMD'
      include 'COMMON.IOUNITS'
      include 'COMMON.FFIELD'
      include 'COMMON.TIME1'
      include 'COMMON.MD'
      integer ilen
      external ilen
c
      call MPI_Barrier(CG_COMM,ierr)
      if (nfgtasks.gt.1) 
     &    call MPI_Bcast(-1,1,MPI_INTEGER,king,FG_COMM,IERROR)
      time1=MPI_WTIME()
c      if (me.eq.king .or. .not. out1file) then
       write (iout,'(a,i4,a)') 'CG processor',me,' is finishing work.'
       write (iout,*) 'Total wall clock time',time1-walltime,' sec'
       if (nfgtasks.gt.1) then
         write (iout,'(80(1h=)/a/(80(1h=)))') 
     &    "Details of FG communication time"
          write (iout,'(7(a40,1pe15.5/),40(1h-)/a40,1pe15.5/80(1h=))') 
     &    "BROADCAST:",time_bcast,"REDUCE:",time_reduce,
     &    "GATHER:",time_gather,
     &    "SCATTER:",time_scatter,"SENDRECV:",time_sendrecv,
     &    "BARRIER ene",time_barrier_e,
     &    "BARRIER grad",time_barrier_g,"TOTAL:",
     &    time_bcast+time_reduce+time_gather+time_scatter+time_sendrecv
     &    +time_barrier
          write (*,*) 'Total wall clock time',time1-walltime,' sec'
          write (*,*) "Processor",me," BROADCAST time",time_bcast,
     &      " REDUCE time",
     &      time_reduce," GATHER time",time_gather," SCATTER time",
     &      time_scatter," SENDRECV",time_sendrecv,
     &      " BARRIER",time_barrier
        endif
c      endif
      write (*,'(a,i4,a)') 'CG processor',me,' is finishing work.'
      if (ilen(tmpdir).gt.0) then
        write (*,*) "Processor",me,
     &   ": moving output files to the parent directory..."
        close(inp)
        close(istat,status='keep')
        if (ntwe.gt.0) call move_from_tmp(statname)
        close(irest2,status='keep')
        if (modecalc.eq.12.or.
     &     (modecalc.eq.14 .and. .not.restart1file)) then
          call move_from_tmp(rest2name) 
        else if (modecalc.eq.14.and. me.eq.king) then
          call move_from_tmp(mremd_rst_name)
        endif
        if (mdpdb) then
         close(ipdb,status='keep')
         call move_from_tmp(pdbname)
        else if (me.eq.king .or. .not.traj1file) then
         close(icart,status='keep')
         call move_from_tmp(cartname)
        endif
        if (me.eq.king .or. .not. out1file) then
          close (iout,status='keep')
          call move_from_tmp(outname)
        endif
      endif
      return
      end
#endif
