CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                              C
C                                N A R E S                                     C
C                                                                              C
C      Program to carry out conformational search of nucleic acids             C
C                in an united-residue approximation                            C
C                                                                              C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      implicit real*8 (a-h,o-z)
      include 'DIMENSIONS'


#ifdef MPI
      include 'mpif.h'
      include 'COMMON.SETUP'
#endif
      include 'COMMON.TIME1'
      include 'COMMON.INTERACT'
      include 'COMMON.NAMES'
      include 'COMMON.GEO'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.CONTACTS'
      include 'COMMON.CHAIN'
      include 'COMMON.VAR'
      include 'COMMON.IOUNITS'
      include 'COMMON.FFIELD'
      include 'COMMON.REMD'
      include 'COMMON.MD'
      include 'COMMON.SBRIDGE'
      double precision hrtime,mintime,sectime
      character*64 text_mode_calc(-2:14) /'test',
     & 'SC rotamer distribution',
     & 'Energy evaluation or minimization',
     & 'Regularization of PDB structure',
     & 'Threading of a sequence on PDB structures',
     & 'Monte Carlo (with minimization) ',
     & 'Energy minimization of multiple conformations',
     & 'Checking energy gradient',
     & 'Entropic sampling Monte Carlo (with minimization)',
     & 'Energy map',
     & 'CSA calculations',
     & 'Not used 9',
     & 'Not used 10',
     & 'Soft regularization of PDB structure',
     & 'Mesoscopic molecular dynamics (MD) ',
     & 'Not used 13',
     & 'Replica exchange molecular dynamics (REMD)'/
      external ilen

      print *,"NARES start"

#ifdef MPI
      call init_task
#else
      call initialize
      call openunits
#endif
      if (me.eq.king)
     & write(iout,*)'### LAST MODIFIED  4/25/08 7:29PM by adam'  
      if (me.eq.king) call cinfo
C Read force field parameters and job setup data
      call readrtns
      call flush(iout)
C
      if (me.eq.king .or. .not. out1file) then
       write (iout,'(2a/)') 
     & text_mode_calc(modecalc)(:ilen(text_mode_calc(modecalc))),
     & ' calculation.' 
       if (minim) write (iout,'(a)') 
     &  'Conformations will be energy-minimized.'
       write (iout,'(80(1h*)/)') 
      endif
      call flush(iout)
C
      if (modecalc.eq.-2) then
        call test
        stop
      else if (modecalc.eq.-1) then
        write(iout,*) "call check_sc_map next"
        call check_bond
        stop
      endif
#ifdef MPI
      if (fg_rank.gt.0) then
C Fine-grain slaves just do energy and gradient components.
        call ergastulum ! slave workhouse in Latin
      else
#endif
      if (modecalc.eq.0) then
        call exec_eeval_or_minim
      else if (modecalc.eq.1) then
        call exec_regularize
      else if (modecalc.eq.3 .or. modecalc .eq.6) then
c        call exec_MC
      else if (modecalc.eq.4) then
        call exec_mult_eeval_or_minim
      else if (modecalc.eq.5) then
        call exec_checkgrad
      else if (ModeCalc.eq.7) then
        call exec_map
      else if (ModeCalc.eq.8) then
        call exec_CSA
      else if (modecalc.eq.11) then
        call exec_softreg
      else if (modecalc.eq.12) then
        call exec_MD
      else if (modecalc.eq.14) then
        call exec_MREMD
      else
        write (iout,'(a)') 'This calculation type is not supported',
     &   ModeCalc
      endif
#ifdef MPI
      endif
C Finish task.
      if (fg_rank.eq.0) call finish_task
c      call memmon_print_usage()
#define DEBUG
#ifdef DEBUG
c       if (fg_rank.gt.0) then
         write (iout,'(80(1h=)/a/(80(1h=)))') 
     &    "Details of FG communication time"
         write (*,'(7(a40,1pe15.5/),40(1h-)/a40,1pe15.5/80(1h=))') 
     &    "BROADCAST:",time_bcast,"REDUCE:",time_reduce,
     &    "GATHER:",time_gather,
     &    "SCATTER:",time_scatter,"SENDRECV:",time_sendrecv,
     &    "BARRIER ene",time_barrier_e,
     &    "BARRIER grad",time_barrier_g,
     &    "TOTAL:",
     &    time_bcast+time_reduce+time_gather+time_scatter+time_sendrecv
         write (*,*) fg_rank,myrank,
     &     ': Total wall clock time',time1-walltime,' sec'
         write (*,*) "Processor",fg_rank,myrank,
     &     ": BROADCAST time",time_bcast," REDUCE time",
     &      time_reduce," GATHER time",time_gather," SCATTER time",
     &      time_scatter," SENDRECV",time_sendrecv,
     &      " BARRIER ene",time_barrier_e,
     &      " BARRIER GRAD",time_barrier_g,
     &      " BCAST7",time_bcast7," BCASTC",time_bcastc,
     &      " BCASTW",time_bcastw,
     &      " TOTAL",
     &      time_bcast+time_reduce+time_gather+time_scatter+
     &      time_sendrecv+time_barrier+time_bcastc
         write (*,*) "Processor",fg_rank,myrank," enecalc",time_enecalc
         write (*,*) "Processor",fg_rank,myrank," sumene",time_sumene
         write (*,*) "Processor",fg_rank,myrank," intfromcart",
     &     time_intfcart
         write (*,*) "Processor",fg_rank,myrank," vecandderiv",
     &     time_vec
         write (*,*) "Processor",fg_rank,myrank," setmatrices",
     &     time_mat
         if (fg_rank.eq.0) then
           write (*,*) "Processor",fg_rank,myrank," lagrangian",
     &       time_lagrangian
           write (*,*) "Processor",fg_rank,myrank," cartgrad",
     &       time_cartgrad," sumgradient",time_sumgradient,
     &       " intcartderiv",time_intcartderiv," inttocart",
     &       time_inttocart
         endif
c      endif
#endif
#undef DEBUG
      call MPI_Finalize(ierr)
      stop 'Bye Bye...'
#else
      call dajczas(tcpu(),hrtime,mintime,sectime)
      stop '********** Program terminated normally.'
#endif
      end
c--------------------------------------------------------------------------
      subroutine exec_MD
      include 'DIMENSIONS'
#ifdef MPI
      include "mpif.h"
#endif
      include 'COMMON.SETUP'
      include 'COMMON.CONTROL'
      include 'COMMON.IOUNITS'
      if (me.eq.king .or. .not. out1file)
     &   write (iout,*) "Calling chainbuild"
      call chainbuild
      call MD
      return
      end
c---------------------------------------------------------------------------
      subroutine exec_MREMD
      include 'DIMENSIONS'
#ifdef MPI
      include "mpif.h"
#endif
      include 'COMMON.SETUP'
      include 'COMMON.CONTROL'
      include 'COMMON.IOUNITS'
      include 'COMMON.REMD'
#ifdef MPI
      if (me.eq.king .or. .not. out1file)
     &   write (iout,*) "Calling chainbuild"
      call chainbuild
      if (me.eq.king .or. .not. out1file)
     &   write (iout,*) "Calling REMD"
      if (remd_mlist) then 
        call MREMD
      else
        do i=1,nrep
          remd_m(i)=1
        enddo
        call MREMD
      endif
#else
      write (iout,*) "MREMD only allowed in parallel mode"
#endif
      return
      end
c---------------------------------------------------------------------------
      subroutine exec_eeval_or_minim
      implicit real*8 (a-h,o-z)
      include 'DIMENSIONS'
#ifdef MPI
      include 'mpif.h'
#endif
      include 'COMMON.SETUP'
      include 'COMMON.TIME1'
      include 'COMMON.INTERACT'
      include 'COMMON.NAMES'
      include 'COMMON.GEO'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.CONTACTS'
      include 'COMMON.CHAIN'
      include 'COMMON.VAR'
      include 'COMMON.IOUNITS'
      include 'COMMON.FFIELD'
      include 'COMMON.REMD'
      include 'COMMON.MD'
      include 'COMMON.SBRIDGE'
      common /srutu/ icall
      double precision energy(0:n_ene)
      double precision energy_long(0:n_ene),energy_short(0:n_ene)
      if (indpdb.eq.0) call chainbuild
#ifdef MPI
      time00=MPI_Wtime()
#endif
      call chainbuild_cart
c      print *,"Processor",myrank," after chainbuild"
c      icall=1
c      call etotal_long(energy_long(0))
c      write (iout,*) "Printing long range energy"
c      call enerprint(energy_long(0))
c      call etotal_short(energy_short(0))
c      write (iout,*) "Printing short range energy"
c      call enerprint(energy_short(0))
c      do i=0,n_ene
c        energy(i)=energy_long(i)+energy_short(i)
c        write (iout,*) i,energy_long(i),energy_short(i),energy(i)
c      enddo
c      write (iout,*) "Printing long+short range energy"
c      call enerprint(energy(0))
      call etotal(energy(0))
#ifdef MPI
      time_ene=MPI_Wtime()-time00
      write (iout,*) "Time for energy evaluation",time_ene
#endif
      print *,"after etotal"
      etota = energy(0)
      etot =etota
      if (minim) then
crc overlap test
        if (overlapsc) then 
          print *, 'Calling OVERLAP_SC'
          call overlap_sc(fail)
        endif 

        if (searchsc) then 
          call sc_move(2,nres-1,10,1d10,nft_sc,etot)
          print *,'SC_move',nft_sc,etot
          write(iout,*) 'SC_move',nft_sc,etot
        endif 

        if (dccart) then
          print *, 'Calling MINIM_DC'
#ifdef MPI
          time1=MPI_WTIME()
#endif
          call minim_dc(etot,iretcode,nfun)
        else
          if (indpdb.ne.0) then 
            call bond_regular
            call chainbuild
          endif
          call geom_to_var(nvar,varia)
          print *,'Calling MINIMIZE.'
#ifdef MPI
          time1=MPI_WTIME()
#endif
          call minimize(etot,varia,iretcode,nfun)
        endif
        print *,'SUMSL return code is',iretcode,' eval ',nfun
#ifdef MPI
        evals=nfun/(MPI_WTIME()-time1)
        print *,'# eval/s',evals
        print *,'refstr=',refstr
#endif
        call etotal(energy(0))
        etot = energy(0)
        call enerprint(energy(0))

        call intout
        call briefout(0,etot)
        if (refstr) call rms_nac_nnc(rms,frac,frac_nn,co,.true.)
          write (iout,'(a,i3)') 'SUMSL return code:',iretcode
          write (iout,'(a,i20)') '# of energy evaluations:',nfun+1
          write (iout,'(a,f16.3)')'# of energy evaluations/sec:',evals
      else
        print *,'refstr=',refstr
        if (refstr) call rms_nac_nnc(rms,frac,frac_nn,co,.true.)
        call enerprint(energy(0))
        call briefout(0,etot)
      endif
      print *,"outpdb ",outpdb
      if (outpdb) call pdbout(etot,titel,ipdb)
      if (outmol2) call mol2out(etot,titel(:32))
      return
      end
c---------------------------------------------------------------------------
      subroutine exec_regularize
      implicit real*8 (a-h,o-z)
      include 'DIMENSIONS'
#ifdef MPI
      include 'mpif.h'
#endif
      include 'COMMON.SETUP'
      include 'COMMON.TIME1'
      include 'COMMON.INTERACT'
      include 'COMMON.NAMES'
      include 'COMMON.GEO'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.CONTACTS'
      include 'COMMON.CHAIN'
      include 'COMMON.VAR'
      include 'COMMON.IOUNITS'
      include 'COMMON.FFIELD'
      include 'COMMON.REMD'
      include 'COMMON.MD'
      include 'COMMON.SBRIDGE'
      double precision energy(0:n_ene)

      call gen_dist_constr
c      call sc_conf
      call intout
      call regularize(nct-nnt+1,etot,rms,cref(1,nnt),iretcode)
      call etotal(energy(0))
      energy(0)=energy(0)-energy(14)
      etot=energy(0)
      call enerprint(energy(0))
      call intout
      call briefout(0,etot)
      if (outpdb) call pdbout(etot,titel,ipdb)
      if (outmol2) call mol2out(etot,titel(:32))
      if (refstr) call rms_nac_nnc(rms,frac,frac_nn,co,.true.)
      write (iout,'(a,i3)') 'SUMSL return code:',iretcode
      return
      end
c---------------------------------------------------------------------------
      subroutine exec_thread
      include 'DIMENSIONS'
#ifdef MP
      include "mpif.h"
#endif
      include "COMMON.SETUP"
c      call thread_seq
      return
      end
c---------------------------------------------------------------------------
      subroutine exec_mult_eeval_or_minim
      implicit real*8 (a-h,o-z)
      include 'DIMENSIONS'
#ifdef MPI
      include 'mpif.h'
      dimension muster(mpi_status_size)
#endif
      include 'COMMON.SETUP'
      include 'COMMON.TIME1'
      include 'COMMON.INTERACT'
      include 'COMMON.NAMES'
      include 'COMMON.GEO'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.CONTACTS'
      include 'COMMON.CHAIN'
      include 'COMMON.VAR'
      include 'COMMON.IOUNITS'
      include 'COMMON.FFIELD'
      include 'COMMON.REMD'
      include 'COMMON.MD'
      include 'COMMON.SBRIDGE'
      double precision varia(maxvar)
      dimension ind(6)
      double precision energy(0:n_ene)
      logical eof
      eof=.false.
#ifdef MPI
      if(me.ne.king) then
        call minim_mcmf
        return
      endif

      close (intin)
      open(intin,file=intinname,status='old')
      write (istat,'(a5,20a12)')"#    ",
     &  (wname(i),i=1,n_ene)
      if (refstr) then
        write (istat,'(a5,20a12)')"#    ",
     &   (ename(i),i=1,n_ene),
     &   "ETOT total","RMSD","nat.contact","nnt.contact"        
      else
        write (istat,'(a5,20a12)')"#    ",
     &    (ename(i),i=1,n_ene),"ETOT total"
      endif

      if (.not.minim) then
        do while (.not. eof)
          if (read_cart) then
            read (intin,'(e15.10,e15.5)',end=1100,err=1100) time,ene
            call read_x(intin,*11)
#ifdef MPI
c Broadcast the order to compute internal coordinates to the slaves.
            if (nfgtasks.gt.1)
     &        call MPI_Bcast(6,1,MPI_INTEGER,king,FG_COMM,IERROR)
#endif
            call int_from_cart1(.false.)
          else
            read (intin,'(i5)',end=1100,err=1100) iconf
            call read_angles(intin,*11)
            call geom_to_var(nvar,varia)
            call chainbuild
          endif
          write (iout,'(a,i7)') 'Conformation #',iconf
          call etotal(energy(0))
          call briefout(iconf,energy(0))
          call enerprint(energy(0))
          etot=energy(0)
          if (refstr) then 
            call rms_nac_nnc(rms,frac,frac_nn,co,.true.)
            write (istat,'(i5,20(f12.3))') iconf,
     &      (energy(i),i=1,n_ene),etot,
     &       rms,frac,frac_nn,co
cjlee end
          else
            write (istat,'(i5,16(f12.3))') iconf,
     &     (energy(i),i=1,n_ene),etot
          endif
        enddo
1100    continue
        goto 1101
      endif

      mm=0
      imm=0
      nft=0
      ene0=0.0d0
      n=0
      iconf=0
c      do n=1,nzsc
      do while (.not. eof)
        mm=mm+1
        if (mm.lt.nodes) then
          if (read_cart) then
            read (intin,'(e15.10,e15.5)',end=11,err=11) time,ene
            call read_x(intin,*11)
#ifdef MPI
c Broadcast the order to compute internal coordinates to the slaves.
            if (nfgtasks.gt.1) 
     &        call MPI_Bcast(6,1,MPI_INTEGER,king,FG_COMM,IERROR)
#endif
            call int_from_cart1(.false.)
          else
            read (intin,'(i5)',end=11,err=11) iconf
            call read_angles(intin,*11)
            call geom_to_var(nvar,varia)
            call chainbuild
          endif
          write (iout,'(a,i7)') 'Conformation #',iconf
          n=n+1
         imm=imm+1
         ind(1)=1
         ind(2)=n
         ind(3)=0
         ind(4)=0
         ind(5)=0
         ind(6)=0
         ene0=0.0d0
         call mpi_send(ind,6,mpi_integer,mm,idint,CG_COMM,
     *                  ierr)
         call mpi_send(varia,nvar,mpi_double_precision,mm,
     *                  idreal,CG_COMM,ierr)
         call mpi_send(ene0,1,mpi_double_precision,mm,
     *                  idreal,CG_COMM,ierr)
c         print *,'task ',n,' sent to worker ',mm,nvar
        else
         call mpi_recv(ind,6,mpi_integer,mpi_any_source,idint,
     *                 CG_COMM,muster,ierr)
         man=muster(mpi_source)
c         print *,'receiving result from worker ',man,' (',iii1,iii,')'
         call mpi_recv(varia,nvar,mpi_double_precision, 
     *               man,idreal,CG_COMM,muster,ierr)
         call mpi_recv(ene,1,
     *               mpi_double_precision,man,idreal,
     *               CG_COMM,muster,ierr)
         call mpi_recv(ene0,1,
     *               mpi_double_precision,man,idreal,
     *               CG_COMM,muster,ierr)
c         print *,'result received from worker ',man,' sending now'

          call var_to_geom(nvar,varia)
          call chainbuild
          call etotal(energy(0))
          iconf=ind(2)
          write (iout,*)
          write (iout,*)
          write (iout,'(a,2i7)') 'Conformation #',iconf,ind(5)

          etot=energy(0)
          call enerprint(energy(0))
          call briefout(it,etot)
c          if (minim) call briefout(it,etot)
          if (refstr) then 
            call rms_nac_nnc(rms,frac,frac_nn,co,.true.)
            write (istat,'(i5,19(f12.3))') iconf,
     &     (energy(i),i=1,n_ene),etot,
     &     rms,frac,frac_nn,co
          else
            write (istat,'(i5,15(f12.3))') iconf,
     &     (energy(i),i=1,n_ene),etot
          endif

          imm=imm-1
          if (read_cart) then
            read (intin,'(e15.10,e15.5)',end=1101,err=1101) time,ene
            call read_x(intin,*11)
#ifdef MPI
c Broadcast the order to compute internal coordinates to the slaves.
            if (nfgtasks.gt.1)
     &        call MPI_Bcast(6,1,MPI_INTEGER,king,FG_COMM,IERROR)
#endif
            call int_from_cart1(.false.)
          else
            read (intin,'(i5)',end=1101,err=1101) iconf
            call read_angles(intin,*11)
            call geom_to_var(nvar,varia)
            call chainbuild
          endif
          n=n+1
          imm=imm+1
          ind(1)=1
          ind(2)=n
          ind(3)=0
          ind(4)=0
          ind(5)=0
          ind(6)=0
          call mpi_send(ind,6,mpi_integer,man,idint,CG_COMM,
     *                  ierr)
          call mpi_send(varia,nvar,mpi_double_precision,man, 
     *                  idreal,CG_COMM,ierr)
          call mpi_send(ene0,1,mpi_double_precision,man,
     *                  idreal,CG_COMM,ierr)
          nf_mcmf=nf_mcmf+ind(4)
          nmin=nmin+1
        endif
      enddo
11    continue
      do j=1,imm
        call mpi_recv(ind,6,mpi_integer,mpi_any_source,idint,
     *               CG_COMM,muster,ierr)
        man=muster(mpi_source)
        call mpi_recv(varia,nvar,mpi_double_precision, 
     *               man,idreal,CG_COMM,muster,ierr)
        call mpi_recv(ene,1,
     *               mpi_double_precision,man,idreal,
     *               CG_COMM,muster,ierr)
        call mpi_recv(ene0,1,
     *               mpi_double_precision,man,idreal,
     *               CG_COMM,muster,ierr)

        call var_to_geom(nvar,varia)
        call chainbuild
        call etotal(energy(0))
        iconf=ind(2)
        write (iout,*)
        write (iout,*)
        write (iout,'(a,2i7)') 'Conformation #',iconf,ind(5)

        etot=energy(0)
        call enerprint(energy(0))
        call briefout(it,etot)
        if (refstr) then 
          call rms_nac_nnc(rms,frac,frac_nn,co,.true.)
          write (istat,'(i5,19(f12.3))') iconf,
     &   (energy(i),i=1,n_ene),etot,
     &   rms,frac,frac_nn,co
        else
          write (istat,'(i5,15(f12.3))') iconf,
     &    (energy(i),i=1,n_ene),etot
        endif
        nmin=nmin+1
      enddo
1101  continue
      do i=1, nodes-1
         ind(1)=0
         ind(2)=0
         ind(3)=0
         ind(4)=0
         ind(5)=0
         ind(6)=0
         call mpi_send(ind,6,mpi_integer,i,idint,CG_COMM,
     *                  ierr)
      enddo
#else
      close (intin)
      open(intin,file=intinname,status='old')
      write (istat,'(a5,20a12)')"#    ",
     &   (wname(i),i=1,n_ene)
      write (istat,'("#    ",20(1pe12.4))')
     &   (weights(i),i=1,n_ene)
      if (refstr) then
        write (istat,'(a5,20a12)')"#    ",
     &   (ename(i),i=1,n_ene),
     &   "ETOT total","RMSD","nat.contact","nnt.contact"
      else
        write (istat,'(a5,14a12)')"#    ",
     &   (ename(i),i=1,n_ene),"ETOT total"
      endif
      do while (.not. eof)
          if (read_cart) then
            read (intin,'(e15.10,e15.5)',end=1100,err=1100) time,ene
            call read_x(intin,*11)
#ifdef MPI
c Broadcast the order to compute internal coordinates to the slaves.
            if (nfgtasks.gt.1)
     &        call MPI_Bcast(6,1,MPI_INTEGER,king,FG_COMM,IERROR)
#endif
            call int_from_cart1(.false.)
          else
            read (intin,'(i5)',end=1100,err=1100) iconf
            call read_angles(intin,*11)
            call geom_to_var(nvar,varia)
            call chainbuild
          endif
        write (iout,'(a,i7)') 'Conformation #',iconf
        if (minim) call minimize(etot,varia,iretcode,nfun)
        call etotal(energy(0))

        etot=energy(0)
        call enerprint(energy(0))
        if (minim) call briefout(it,etot) 
        if (refstr) then 
          call rms_nac_nnc(rms,frac,frac_nn,co,.true.)
          write (istat,'(i5,18(f12.3))') iconf,
     &   (energy(i),i=1,n_ene),
     &   etot,rms,frac,frac_nn,co
cjlee end
        else
          write (istat,'(i5,14(f12.3))') iconf,
     &   (energy(i),i=1,n_ene),etot
        endif
      enddo
   11 continue
 1100 continue
#endif
      return
      end
c---------------------------------------------------------------------------
      subroutine exec_checkgrad
      implicit real*8 (a-h,o-z)
      include 'DIMENSIONS'
#ifdef MPI
      include 'mpif.h'
#endif
      include 'COMMON.SETUP'
      include 'COMMON.TIME1'
      include 'COMMON.INTERACT'
      include 'COMMON.NAMES'
      include 'COMMON.GEO'
      include 'COMMON.HEADER'
      include 'COMMON.CONTROL'
      include 'COMMON.CONTACTS'
      include 'COMMON.CHAIN'
      include 'COMMON.VAR'
      include 'COMMON.IOUNITS'
      include 'COMMON.FFIELD'
      include 'COMMON.REMD'
      include 'COMMON.MD'
      include 'COMMON.SBRIDGE'
      common /srutu/ icall
      double precision energy(0:n_ene)
      if (indpdb.eq.0) call chainbuild
      usampl=.true.
      totT=1.d0
      eq_time=0.0d0
      call chainbuild_cart
      call cartprint
      call intout
      icall=1
      call etotal(energy(0))
      etot = energy(0)
      call enerprint(energy(0))
      write (iout,*) "Uconst",Uconst," Uconst_back",uconst_back
      print *,'icheckgrad=',icheckgrad
      goto (10,20,30) icheckgrad
  10  call check_ecartint
      return
  20  call check_cartgrad
      return
  30  call check_eint
      return
      end
c---------------------------------------------------------------------------
      subroutine exec_map
C Energy maps
      call map_read
      call map
      return
      end
c---------------------------------------------------------------------------
      subroutine exec_CSA
#ifdef MPI
      include "mpif.h"
#endif
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
C Conformational Space Annealling programmed by Jooyoung Lee.
C This method works only with parallel machines!
#ifdef MPI
      call together
#else
      write (iout,*) "CSA works on parallel machines only"
#endif
      return
      end
c---------------------------------------------------------------------------
      subroutine exec_softreg
      include 'DIMENSIONS'
      include 'COMMON.IOUNITS'
      include 'COMMON.CONTROL'
      double precision energy(0:n_ene)
      call chainbuild
      call etotal(energy(0))
      call enerprint(energy(0))

      call softreg

      call etotal(energy(0))
      etot=energy(0)
      call enerprint(energy(0))
      call intout
      call briefout(0,etot)
      if (refstr) call rms_nac_nnc(rms,frac,frac_nn,co,.true.)
      return
      end
